//Test of cheap 13.56 Mhz RFID-RC522 module from eBay
//This code is based on Martin Olejar's MFRC522 library. Minimal changes
//Adapted for Nucleo STM32 F401RE. Should work on other Nucleos too

//Connect as follows:
//RFID pins        ->  Nucleo header CN5 (Arduino-compatible header)
//----------------------------------------
//RFID IRQ=pin5    ->   Not used. Leave open
//RFID MISO=pin4   ->   Nucleo SPI_MISO=PA_6=D12
//RFID MOSI=pin3   ->   Nucleo SPI_MOSI=PA_7=D11
//RFID SCK=pin2    ->   Nucleo SPI_SCK =PA_5=D13
//RFID SDA=pin1    ->   Nucleo SPI_CS  =PB_6=D10
//RFID RST=pin7    ->   Nucleo         =PA_9=D8
//3.3V and Gnd to the respective pins                              
                              
#include "mbed.h"
#include "MFRC522.h"
#include "TextLCD.h"

// Nucleo Pin for MFRC522 reset (pick another D pin if you need D8)
#define MF_RESET    p15
#define SPI_MOSI    p5
#define SPI_MISO    p6
#define SPI_SCK     p7
#define SPI_CS      p8

DigitalOut LedGreen(LED1);

//LCD text display status
TextLCD lcd(p10, p11, p27, p28, p29, p30); // rs, e, d4-d7

//Serial connection to PC for output
Serial pc(USBTX, USBRX);

MFRC522    RfChip   (SPI_MOSI, SPI_MISO, SPI_SCK, SPI_CS, MF_RESET);

unsigned char tagRX[20];            //array to store incoming tag
int tagRXindex=0;                   //keeps track of the index in tagRX
bool tagIncomingFlag=false;         //flag to indicate that a new tag is busy being read
bool newtagFlag=false;              //flag to indicate that a new tag was finished being read

int main(void) {
  lcd.printf("starting...\r\n");

  // Init. RC522 Chip
  RfChip.PCD_Init();

    lcd.printf("init passed\r\n");


  while (true) {
    LedGreen = 1;

    // Look for new cards
    if ( ! RfChip.PICC_IsNewCardPresent())
    {
      wait_ms(500);
      continue;
    }

    // Select one of the cards
    if (!RfChip.PICC_ReadCardSerial())
    {
      wait_ms(500);
      lcd.printf("card read\r\n");
      continue;
    }

    LedGreen = 0;

    // Print Card UID - original
    //lcd.printf("Card UID: ");
    //for (uint8_t i = 0; i < RfChip.uid.size; i++)
    //{
    //  lcd.printf(" %X02", RfChip.uid.uidByte[i]);
    //}
    //lcd.printf("\n\r");
    
    // Print Card UID - mario
    //lcd.cls();
    //lcd.printf("Card UID: ");
    for (uint8_t i = 0; i < RfChip.uid.size; i++)
    {
        tagRX[i] = RfChip.uid.uidByte[i];
    }
    lcd.printf("\n\r");

    // Print Card type
    //uint8_t piccType = RfChip.PICC_GetType(RfChip.uid.sak);
    //lcd.printf("PICC Type: %s \n\r", RfChip.PICC_GetTypeName(piccType));
    
    lcd.cls();
    lcd.printf("Card UID: ");
    lcd.printf(" %s", tagRX);
    
    wait_ms(1000);
  }
}